<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Payment\DTO;

use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\OAuth\AccessToken\ApplicationContextProvider;
use Invertus\ConnectPay\OAuth\AuthNonce;
use Invertus\ConnectPay\Payment\Service\CallBackLinkService;
use Invertus\ConnectPay\Payment\Service\SignatureService;
use Language;
use Context;
use Order;
use Customer;
use Address;
use Country;
use Currency;
use Invertus\ConnectPay\Adapter\Tools;

class InitializeApiRequest
{
    private const CONTENT_TYPE = 'application/json';
    private const ACCEPT = 'application/json;version=2';
    private const MANDATORY_PAYMENT_METHOD = 'PIS';
    private const MANDATORY_PAYMENT_METHOD_CARD = 'CARD';
    private const REQUEST_METHOD = 'post';

    private $applicationContext;
    private $authNonce;
    private $context;
    private $callBackLinkService;
    private $order;
    private $tools;
    private $signatureService;
    private $configuration;

    public function __construct(
        ApplicationContextProvider $applicationContextProvider,
        AuthNonce $authNonce,
        CallBackLinkService $callBackLinkService,
        Tools $tools,
        SignatureService $signatureService,
        Configuration $configuration
    ) {
        $this->applicationContext = $applicationContextProvider->get();
        $this->authNonce = $authNonce;
        $this->context = Context::getContext();
        $this->callBackLinkService = $callBackLinkService;
        $this->tools = $tools;
        $this->signatureService = $signatureService;
        $this->configuration = $configuration;

        $orderId = Order::getOrderByCartId($this->context->cart->id);
        $this->order = new Order($orderId);
    }

    public function getRequestData(string $requestUrl, string $providerCode): array
    {
        $billingAddressId = (int)$this->order->id_address_delivery ?? 0;
        $shippingAddressId = (int)$this->order->id_address_invoice ?? 0;
        
        $data = [
            'merchant' => $this->getMerchantData(),
            'payment' => $this->getPaymentData($providerCode),
            'identifiers' => $this->getIdentifiersData(),
            'consumer' => $this->getConsumerData(),
        ];
        
        if (!empty($this->getAddressData($billingAddressId))) {
          $data['billingAddress'] = $this->getAddressData($billingAddressId);
        }
        
        if ($billingAddressId != $shippingAddressId) {
          $data['shippingAddress'] = $this->getAddressData($shippingAddressId);
        }

        $requestBody = json_encode($data);
        
	    return [
	        'headers' => [
	            'Authorization' => $this->getAuthorizationKey(),
	            'Content-Type' => self::CONTENT_TYPE,
	            'X-Request-ID' => $this->authNonce->generateNonce(),
	            'Accept' => self::ACCEPT,
	            'X-Signature' => $this->signatureService->generateSignature(
	                self::REQUEST_METHOD,
	                $requestUrl,
	                $requestBody
	            ),
	        ],
	        'body' => $requestBody,
	    ];
    }

    private function getMerchantData(): array
    {
        $language = (
        in_array(
            Language::getIsoById($this->context->cart->id_lang),
            ConnectPayConfig::LIST_OF_SUPPORTED_LANGUAGES
        )
        )? Language::getIsoById($this->context->cart->id_lang) : ConnectPayConfig::DEFAULT_LANGUAGE;

	    return [
	        'brandId' => $this->applicationContext->getBrandId(),
	        'redirectUrl' => $this->callBackLinkService->getPaymentCallBackLink(),
	        'language' => $language,
	    ];
    }

    private function getPaymentData(string $providerCode): array
    {
        $totalPaid = (float) $this->order->total_paid;
        $amount = number_format((float)$totalPaid, 2, '.', '');
        $paymentMethod = $providerCode == 'by_card' ? self::MANDATORY_PAYMENT_METHOD_CARD : self::MANDATORY_PAYMENT_METHOD;
        $currency_iso_code = $this->getCurrencyIsoCode();

        $paymentData = [
            'paymentMethod' => $paymentMethod,
            'instructedAmount' => [
                'amount' => $amount,
                'currency' => $currency_iso_code,
            ],
            'remittanceInformationUnstructured' => $this->order->id,
        ];


        if (!empty($providerCode) && $providerCode !== 'by_card') {
            $paymentData['providerId'] = $providerCode;
        }

        if (empty($providerCode)) {
            $iso_code = $this->configuration->get(ConnectPayConfig::SELECT_PROVIDERS_PLACEHOLDER, $this->context->shop->id);
        } else {
            $address = new Address($this->context->cart->id_address_delivery);
            $deliveryAddressCountry = new Country($address->id_country);
            $iso_code = $this->tools->strtoupper($deliveryAddressCountry->iso_code);
        }

        if (in_array($iso_code, ConnectPayConfig::LIST_OF_SUPPORTED_COUNTRIES)) {
            $paymentData['providerCountryCode'] = $iso_code;
        } else {
            $paymentData['providerCountryCode'] = $this->tools->strtoupper(ConnectPayConfig::DEFAULT_COUNTRY);
        }

        return $paymentData;
    }
    
    private function getAddressData(int $addressId): array
    {
      $address = new Address($this->context->cart->id_address_delivery);
      $addressCountry = new Country($address->id_country);
      
      $data = [];
      
      if (!empty($address->address1)) {
        $data['addressLine1'] = $this->adjustInputField($address->address1);
      }
      
      if (!empty($address->address2)) {
        $data['addressLine2'] = $this->adjustInputField($address->address2);
      }
      
      if (!empty($address->city)) {
        $data['city'] = $this->adjustInputField($address->city);
      }
      
      if (!empty($addressCountry->iso_code)) {
        $data['countryCode'] = $this->tools->strtoupper($addressCountry->iso_code);
      }
      
      if (!empty($address->postcode)) {
        $data['postcode'] = $address->postcode;
      }
      
      return $data;
    }
    
    /**
     * Trims text input field.
     */
    private function adjustInputField(string $text): string
    {
      $maxCharacters = 100;
      $start = 0;
      
      $adjustedText = strlen($text) > $maxCharacters ? substr($text, $start, $maxCharacters) : $text;
      
      return $adjustedText;
    }

    private function getIdentifiersData(): array
    {
	    return [
	        'merchantReference' => $this->order->reference,
	        'endToEndId' => $this->authNonce->generateNonce(),
	    ];
    }

    private function getConsumerData(): array
    {
        $customer = new Customer($this->order->id_customer);

	    return [
	        'firstName' => $customer->firstname,
	        'lastName' => $customer->lastname,
	        'ipAddress' => $this->tools->getRemoteAdd(),
	        'email' => $customer->email,
	    ];
    }

    private function getAuthorizationKey(): string
    {
        $encodedKey = base64_encode(
            $this->applicationContext->getApplicationKey() .
            ':' .
            $this->applicationContext->getApplicationSecret()
        );

        return 'Basic ' . $encodedKey;
    }

    private function getCurrencyIsoCode(): string
    {
        $currency_id = $this->order->id_currency;
        $currency = Currency::getCurrency($currency_id);

        return $currency['iso_code'];
    }
}
